/******************************* cgmparse.c *********************************/

#include <string.h>
#include <dir.h>
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include "cgmparse.h"
#include "cgmasm.h"


char cgm_cbuffer[MAXCGMCHAR];  /* character buffer */
FILE  *cgmsrc = NULL;          /* cgm source file pointer */

int cgm_parse_error = 0;      /* error flag */
struct cgm_head cgm_header;   /* cgm element header */
int cgm_done = 0;             /* done flag */
int cgm_color_mode_in = 0;    /* indexed/direct color flag */
int cgm_color_index_precision = 8;
int cgm_color_precision = 8;



void cgm_read_element_header(void)
{ /* fills the cgm_header structure */

    int  temp;

    temp = _cgm_oneival();
    cgm_header.partition = 0;
    cgm_header.odd = 0;
    cgm_header.class = (temp & 0xf000) >> 12;    
    cgm_header.id = (temp & 0x0fe0) >> 5;    
    cgm_header.length = (temp & 0x001f);    
    if (cgm_header.length != 31)
    {  /* short form command header */
        if (( cgm_header.length & 0x0001) == 1)
            cgm_header.length++; /* add padded zero byte to make even */
    }
    else
    {  /* extended form command header */
        cgm_header.length = _cgm_oneival(); /* length in bytes */

        if ( cgm_header.length & 0x8000)
        {
            cgm_header.partition = 1;
            cgm_header.length &= 0x7fff;
            if (cgm_header.length & 0x0001) cgm_header.odd = 1;
        }
        else if (cgm_header.length & 0x0001) cgm_header.length++;
    }
}


unsigned int cgm_read_string(void)
{  /* Fills cgm_cbuffer[] with a string. */
   /* Returns the number of characters in the string */
   /* (excluding the trailing null character). */

   unsigned int nchar;
   unsigned char temp;
   fread( &temp, 1, 1, cgmsrc);   /* read string length */
   nchar = temp;
   if(nchar == 255)
   { /* extended length */
      nchar = cgm_oneival();
   }
   if ( nchar >= MAXCGMCHAR)
      nchar = MAXCGMCHAR -1; 
   fread( &cgm_cbuffer, 1, nchar,cgmsrc);   /* read nchar chars */
   cgm_cbuffer[nchar] = '\0'; /* include trailing null character */ 
   cgm_header.length -= (nchar +1);
   return(nchar);
}

int cgm_skipdata(void)
{ /* skips to the next command header */
  /* returns 0 on error, and 1 on success */

    int lpart,num;
    char WastBasket[256];
    lpart = cgm_header.partition;
    num = cgm_header.length;
    do
    {  
     start: 
         if ( (cgm_header.odd) && !lpart) num++;
         while( num > 0)
         {
             if( num > 256)
             {
                if(fread( WastBasket, 1, 256,cgmsrc) != 256)
                {
                    cgm_header.length = 0;
                    return(0);
                }
                num -= 256;
                continue;
             }
             if(fread( WastBasket, 1, num,cgmsrc) != num)
             {
                 cgm_header.length = 0;
                 return(0);
             }
             num = 0;

         }
         if(lpart)
         {
              num = _cgm_oneival();
              if ( num & 0x8000)
              {
                   lpart = 1;
                   num &= 0x7fff;
                   cgm_header.odd = cgm_header.odd ^ (num & 0x0001);
              }
              else
              {
                  lpart = 0;
                  goto start;
              }
         }
    } while (lpart);
    cgm_header.length = 0;
    if (ferror(cgmsrc) || feof(cgmsrc) )
       return(0);
    return(1);
}


unsigned int cgm_read_integers(unsigned maxsize,int *buffer)
{  /* fills buffer[], maxsize is maximum number integers to read */
   /* buffer size limited to 64k */
   /* returns the number of integers read */

    unsigned int size,num,temp,k;
    num = 2*maxsize;
    size = 0;
    k = 0;
    while(1)
    {
         if (cgm_header.length == 0)
         {
            if (cgm_header.partition != 1)
                break;
            cgm_header.length = _cgm_oneival();
            if ( cgm_header.length & 0x8000)
            {
                cgm_header.partition = 1;
                cgm_header.length &= 0x7fff;
                cgm_header.odd = cgm_header.odd ^ (cgm_header.length & 0x0001);
            }
            else
                cgm_header.partition = 0;
         }

         if ( num < cgm_header.length)
         {
            temp = fread(&buffer[k],1,num,cgmsrc);
            if(temp != num)
            {
               cgm_parse_error = 1;
               return(0);
            }
         }
         else
         {
            temp = fread(&buffer[k],1,cgm_header.length,cgmsrc);
            if(temp != cgm_header.length)
            {
               cgm_parse_error = 1;
               return(0);
            }
         }
         num -= temp;
         cgm_header.length -= temp;
         temp = temp/2;
         size += temp;
         k += temp;
         if ( num == 0)
            break;    
    }
    cgm_swab(size,buffer);
    return(size);  
}

int cgm_onebval(void)
{
    int temp;
    temp = 0;
    cgm_header.length--;
    fread(&temp,1,1,cgmsrc);
    return(temp);
}


int cgm_read_color_index(void)
{
    if( cgm_color_index_precision == 8)
    {
        if( cgm_header.length <= 0)
            return(-1);
        return(cgm_onebval());

    }
    else
    {
        if( cgm_header.length < 2)
            return(-1);
        return(cgm_oneival());
    }
}
int cgm_read_rgb_color(int *rgb)
{

    if( cgm_color_precision == 8)
    {
        if( cgm_header.length < 3)
            return(-1);

         rgb[0] = cgm_onebval();
         rgb[1] = cgm_onebval();
         rgb[2] = cgm_onebval();
         return(0);
    }
    else
    {
        if( cgm_header.length < 6)
            return(-1);
         rgb[0] = cgm_oneival();
         rgb[1] = cgm_oneival();
         rgb[2] = cgm_oneival();
         return(0);
    }


}







